@echo off
REM
REM Script  : esilo_backup_baremetal_windows_xp_2003.cmd
REM
REM Version : 1.1.0
REM  
REM Purpose : Backup Windows bare metal recovery files to
REM           a folder and prepare the folder to be backed
REM           up by the eSilo Backup program
REM
REM Backup Requires:
REM 
REM 1. Windows XP SP3 (32-bit only), or
REM    Windows 2000, or
REM    Windows Server 2003
REM 2. The ntbackup.exe program (included with Windows)
REM 3. Sufficient free disk space to write BFK file
REM 4. Virtual floppy driver: vfd.dll, vfd.exe, vfd.sys
REM 5. Tee program: wtee.exe
REM
REM Restore Requires:
REM
REM 1. A bootable CD of the Windows installer matching the 
REM    version you have installed.
REM
REM 2. A floppy disk for the ASR files / Emergency Repair Disk
REM
REM Setup:
REM 
REM 1. Extract the esilo_backup_baremetal_windows_xp_2003.zip
REM    file to a folder where you will be writing the backup to.
REM    For example, E:\eSiloBackupFiles\
REM
REM 2. Install the virtual floppy driver included in the ZIP file.
REM    Use the vfdwin.exe program to install the driver,
REM    start it, and set it to run as 'auto(matic)'
REM
REM 3. Adjust the settings in the configuration section of this file
REM    (below) as needed.
REM
REM 4. For Windows 2000: Manually run ntbackup to create an Emergency Repair Disk.
REM    (Enable the "Also backup the registry to the repair directory..." option).
REM    Then copy the files from the ERD as well as registry files from:
REM    %SystemRoot%\Repair\RegBack
REM    to the "Floppy" subdirectory of the script folder.
REM
REM 5. Configure eSilo Backup to run this file as a pre-backup script
REM    and enable the "Abort backup if pre-backup command fails" option.
REM
REM Restore Steps (XP or 2003):
REM
REM 1. Use eSilo Backup to restore the files in the Floppy subfolder
REM    to a floppy disk.
REM
REM 2. Use eSilo Backup to restore the BKF file to a location that
REM    will be accessible from the machine you are restoring to.
REM    This could be a local disk, external USB drive, or network path.
REM
REM 3. Boot from the Windows installation media, press F2 to enter ASR.
REM
REM 4. Insert the ASR floppy when prompted.
REM
REM 5. Setup will install Windows. Let it finish.
REM
REM 6. When prompted, select your BKF file.
REM
REM 7. Windows will perform the restore and reboot.
REM
REM Restore Steps (Windows 2000):
REM
REM Windows 2000 must be re-installed manually and updated to the same OS version.
REM Then use ntbackup to restore using the BKF file.
REM Alternatively, you may create an Emergency Repair Disk (using files in the
REM "Floppy" directory) to repair a system that does not boot.
REM
REM Notes:
REM
REM 1. You may use NTFS compression on the DESTINATION_FOLDER to
REM    reduce the real disk space requirement.
REM
REM 2. If disk space runs out while NTbackup is running, it may hang.
REM    The only remedy is to restart the machine and free up space.
REM
REM 3. If you already have a floppy drive using drive A:, you will need
REM    to either permanently disable that drive from within device manager or
REM    set FLOPPY_DRIVE to an available drive (such as B:)
REM
REM Further reading:
REM
REM Automated System Recovery overview in Windows XP:
REM <http://support.microsoft.com/kb/818903>
REM
REM How ASR Works (Windows XP/2003):
REM <http://technet.microsoft.com/en-us/library/cc758365%28v=ws.10%29.aspx>
REM
REM Using the Emergency Repair Disk (Windows 2000)
REM <http://technet.microsoft.com/en-us/library/cc938529.aspx>



REM ******************* BEGIN CONFIGURATION ******************* 


REM The destination folder for backup files
REM This is also where the script files are located.
REM You should select this entire folder in eSilo Backup.
REM
set DESTINATION_FOLDER=C:\eSiloBackupScripts

REM The name of the backup file itself (use bkf extension)
REM
set BACKUP_FILENAME=FullSystemBackup.bkf

REM Verify the backup? Possible values: "no" or "yes"
REM
set VERIFY=no

REM Use virtual floppy driver? Possible values: "yes" or "no"
REM
set VIRTUAL_FLOPPY=yes

REM Floppy drive letter to use
REM
set FLOPPY_DRIVE=B:

REM For Windows 2000 only: List of the volumes to backup (space-separated)
REM The volumes should be entered as:
REM "C:\" for the entire C drive,
REM "D:\data" for a specific folder, and 
REM "SystemState" to include the system state.
REM
set ITEMS_TO_BACKUP=C:\ SystemState

REM ******************* END CONFIGURATION ******************* 







REM Set some variables

SET TEE=%DESTINATION_FOLDER%\wtee.exe
SET LOG=%DESTINATION_FOLDER%\backup.log
set FLOPPY_DIR=%DESTINATION_FOLDER%\Floppy


REM Verify Windows Version
echo:>%LOG%
echo %DATE% %TIME% ] eSilo Windows Server Backup Script Started. | %TEE% -a %LOG%
echo Checking Windows version... | %TEE% -a %LOG%

ver | findstr /B /C:"Microsoft Windows XP [Version 5.1" >nul
if "%ERRORLEVEL%" == "0" goto os_xp

ver | findstr /B /C:"Microsoft Windows [Version 5.2" >nul
if "%ERRORLEVEL%" == "0" goto os_2003

ver | findstr /B /C:"Microsoft Windows 2000 [Version 5.00" >nul
if "%ERRORLEVEL%" == "0" goto os_2k

goto os_bad

:os_xp
set OS=xp
set NTBACKUP_CMD=asrbackup
set NTBACKUP_OPTS=/snap:on
goto os_ok

:os_2003
set OS=2003
set NTBACKUP_CMD=asrbackup
set NTBACKUP_OPTS=/snap:on
goto os_ok

REM Windows 2000 requires special handling
:os_2k
set OS=2k
set NTBACKUP_CMD=backup
set NTBACKUP_OPTS=@%DESTINATION_FOLDER%\BackupSelection.bks
echo. > %DESTINATION_FOLDER%\BackupSelection-ANSI.bks
for %%x in ( %ITEMS_TO_BACKUP% ) do ( 
@echo %%x >> %DESTINATION_FOLDER%\BackupSelection-ANSI.bks
)
CMD /U /C Type %DESTINATION_FOLDER%\BackupSelection-ANSI.bks > %DESTINATION_FOLDER%\BackupSelection.bks
goto os_ok

:os_bad

echo ERROR: Unknown or unsupported operating system version: | %TEE% -a %LOG%
ver | %TEE% -a %LOG%

eventcreate /l Application /t ERROR /so "eSilo Windows XP/2003 Bare Metal Backup Script" /id 1 /d "eSilo Windows XP/2003 Bare Metal Backup Script Failed: Invalid Operating System Version" 2>&1 | %TEE% -a %LOG%

exit /b 1

:os_ok
echo Operating system version check: OK (%OS%) | %TEE% -a %LOG%


cd /D %DESTINATION_FOLDER%

REM Ensure the floppy directory exists
mkdir %FLOPPY_DIR% | %TEE% -a %LOG%

IF %VIRTUAL_FLOPPY%. == yes. goto floppy_vfd
goto floppy_regular
:floppy_vfd
echo Mounting virtual floppy... | %TEE% -a %LOG%

REM Close and unlink in case it already exists
vfd.exe close %FLOPPY_DRIVE% /FORCE | %TEE% -a %LOG%
vfd.exe ulink %FLOPPY_DRIVE% | %TEE% -a %LOG%

REM Link and open new
vfd.exe link 0 %FLOPPY_DRIVE% | %TEE% -a %LOG%
vfd.exe open %FLOPPY_DRIVE% /RAM /NEW /FORCE /1.44 | %TEE% -a %LOG%
%SystemRoot%\system32\ping.exe -r 3 -w 1000 127.0.0.1 >nul
:floppy_regular

REM This runs the ASR backup
echo Running ntbackup...| %TEE% -a %LOG%
echo %SystemRoot%\system32\ntbackup.exe %NTBACKUP_CMD% %NTBACKUP_OPTS% /n "ASR for %COMPUTERNAME%" /d "ASR for %COMPUTERNAME%" /v:%VERIFY% /r:no /rs:no /hc:off /m normal /j "FullSystemBackup" /l:s /f %DESTINATION_FOLDER%\%BACKUP_FILENAME% | %TEE% -a %LOG%
%SystemRoot%\system32\ntbackup.exe %NTBACKUP_CMD% %NTBACKUP_OPTS% /n "ASR for %COMPUTERNAME%" /d "ASR for %COMPUTERNAME%" /v:%VERIFY% /r:no /rs:no /hc:off /m normal /j "FullSystemBackup" /l:s /f %DESTINATION_FOLDER%\%BACKUP_FILENAME%


if %ERRORLEVEL%. == 0. goto ntb_ok
echo ntbackup failed with error %ERRORLEVEL% | %TEE% -a %LOG%
IF %VIRTUAL_FLOPPY%. == yes. goto floppy_vfd3
goto floppy_regular3
:floppy_vfd3
echo Unmounting virtual floppy... | %TEE% -a %LOG%
vfd.exe close %FLOPPY_DRIVE% /FORCE | %TEE% -a %LOG%
vfd.exe ulink %FLOPPY_DRIVE% | %TEE% -a %LOG%
:floppy_regular3

exit /b 1

:ntb_ok
echo ntbackup finished with no error. Waiting a few seconds... | %TEE% -a %LOG%
%SystemRoot%\system32\ping.exe -r 3 -w 1000 127.0.0.1 >nul

REM Windows 2000 does not use ASR files. Do alternative check for ERD files.
if "%OS%" == "2k" goto asr_2k_check
goto asr_regular_check

:asr_2k_check
if not exist %FLOPPY_DIR%\autoexec.nt goto asr_missing2
if not exist %FLOPPY_DIR%\config.nt goto asr_missing2
if not exist %FLOPPY_DIR%\setup.log goto asr_missing2
if not exist %FLOPPY_DIR%\default goto asr_missing2
if not exist %FLOPPY_DIR%\NTUSER.DAT goto asr_missing2
if not exist %FLOPPY_DIR%\SAM goto asr_missing2
if not exist %FLOPPY_DIR%\SECURITY goto asr_missing2
if not exist %FLOPPY_DIR%\Software goto asr_missing2
if not exist %FLOPPY_DIR%\System goto asr_missing2
if not exist %FLOPPY_DIR%\UsrClass.dat goto asr_missing2

echo The Emergency Repair Disk (ERD) files exist. | %TEE% -a %LOG%
goto asr_ok
:asr_missing2
echo.
echo One or more Emergency Repair Disk files are missing. | %TEE% -a %LOG%
echo Please use ntbackup to manually create an E.R.D. and copy the files to '%FLOPPY_DIR%' | %TEE% -a %LOG%
echo The files are:
echo autoexec.nt, config.nt, and setup.log (written to the floppy)  | %TEE% -a %LOG%
echo default, NTUSER,DAT, SAM, SECURITY, Software, System, and UsrClass.dat (written to %WINDIR%\repair\RegBack)  | %TEE% -a %LOG%
exit /b 5

:asr_regular_check
REM Check for ASR files
if not exist %FLOPPY_DRIVE%\asr.sif goto asr_missing1
if not exist %FLOPPY_DRIVE%\asrpnp.sif goto asr_missing1
goto asr_ok1

:asr_missing1
echo ERROR: One or more ASR files were not created by ntbackup. | %TEE% -a %LOG%
exit /b 4

:asr_ok1
echo ASR files seem to have been created successfully. | %TEE% -a %LOG%

REM The line below copies over the required floppy disk files to compete the backup.
echo Copying floppy files... | %TEE% -a %LOG%
copy /Y /V %FLOPPY_DRIVE%\asr*.sif %FLOPPY_DIR% | %TEE% -a %LOG%
set COPY_ERROR=%ERRORLEVEL%


IF %VIRTUAL_FLOPPY%. == yes. goto floppy_vfd2
goto floppy_regular2
:floppy_vfd2
echo Unmounting virtual floppy... | %TEE% -a %LOG%
vfd.exe close %FLOPPY_DRIVE% /FORCE | %TEE% -a %LOG%
vfd.exe ulink %FLOPPY_DRIVE% | %TEE% -a %LOG%
:floppy_regular2

if %COPY_ERROR%. == 0. goto copy_ok

echo Copy failed. | %TEE% -a %LOG%
exit /b 2

:copy_ok

if not exist %FLOPPY_DIR%\asr.sif goto asr_missing
if not exist %FLOPPY_DIR%\asrpnp.sif goto asr_missing
echo ASR files copied successfully. | %TEE% -a %LOG%
goto asr_ok

:asr_missing
echo ERROR: One or more ASR file were not copied! | %TEE% -a %LOG%
exit /b 3

:asr_ok

echo Script completed Successfully. | %TEE% -a %LOG%
